searchNodes=[{"ref":"EctoSearcher.html","title":"EctoSearcher","type":"module","doc":"EctoSearcher is an attempt to bring dynamicly built queries (hello Ransack) to the world of Ecto. Installation Add ecto_searcher from github to your mix.ex deps: def deps do [ {:ecto_searcher, &quot;~&gt; 0.2.0&quot;} ] end Notice This package is build for sql queries and is tested with PostgreSQL. Every other usage may not work. Usage (short) EctoSearcher.Searcher — use this to search EctoSearcher.Sorter — use this to sort EctoSearcher.Mapping — use this to implement custom searches EctoSearcher.Mapping.Default — use this if you don&#39;t want to implement custom mapping Usage Obviously, EctoSearcher works on top of ecto schemas and ecto repos. It consumes ecto query and adds conditions built from input. Searching with EctoSearcher.Searcher.search/5 and sorting EctoSearcher.Searcher.sort/5 could be used separately or together. Searching To search use EctoSearcher.Searcher.search/4 or EctoSearcher.Searcher.search/5: Basic usage: defmodule TotallyNotAPhoenixController do def not_some_controller_method() do base_query = Ecto.Query.from(MyMegaModel) search = %{&quot;name_eq&quot; =&gt; &quot;Donald Trump&quot;, &quot;description_cont&quot; =&gt; &quot;My president&quot;} query = EctoSearcher.Searcher.search(base_query, MyMegaModel, search) MySuperApp.Repo.all(query) end end Sorting To sort use EctoSearcher.Sorter.sort/3 or EctoSearcher.Sorter.sort/5: defmodule TotallyNotAPhoenixController do def not_some_controller_method() do base_query = Ecto.Query.from(MyMegaModel) sort = %{&quot;field&quot; =&gt; &quot;name&quot;, &quot;order&quot; =&gt; &quot;desc&quot;} query = EctoSearcher.Sorter.sort(base_query, MyMegaModel, sort) MySuperApp.Repo.all(query) end end Custom fields and matchers In case you need to implement custom field queries or custom matchers you can implement custom Mapping (using EctoSearcher.Mapping behaviour): defmodule MySuperApp.CustomMapping do use EctoSearcher.Mapping def matchers do custom_matchers = %{ &quot;not_eq&quot; =&gt; fn field, value -&gt; Query.dynamic([q], ^field != ^value) end } ## No magic, just plain data manipulation Map.merge( custom_matchers, EctoSearcher.Mapping.Default.matchers() ) end def fields do %{ datetime_field_as_date: %{ query: Query.dynamic([q], fragment(&quot;?::date&quot;, q.custom_field)), type: :date } } end end And use it in EctoSearcher.Searcher.search/5 or EctoSearcher.Sorter.sort/5: defmodule TotallyNotAPhoenixContext do import Ecto.Query require Ecto.Query def not_some_context_method() do search = %{ &quot;name_eq&quot; =&gt; &quot;Donald Trump&quot;, &quot;datetime_as_date_gteq&quot; =&gt; &quot;2016-11-08&quot;, &quot;datetime_as_date_lteq&quot; =&gt; &quot;2019-09-11&quot;, &quot;description_not_eq&quot; =&gt; &quot;Not my president&quot; } sort = %{ &quot;field&quot; =&gt; &quot;datetime_as_date_gteq&quot;, &quot;order&quot; =&gt; &quot;desc&quot; } base_query = from(q in MyMegaModel, where: [q.id &lt; 1984]) query = base_query |&gt; EctoSearcher.Searcher.search(MyMegaModel, search, MySuperApp.CustomMapping) |&gt; EctoSearcher.Searcher.search(base_query, MyMegaModel, sort, MySuperApp.CustomMapping) |&gt; MySuperApp.Repo.all() end end EctoSearcher.Searcher.search/5 and EctoSearcher.Sorter.sort/5 looks up fields in mapping first, then looks up fields in schema."},{"ref":"EctoSearcher.Mapping.html","title":"EctoSearcher.Mapping","type":"behaviour","doc":"Behaviour for search query, matcher and field mappings Usage Either adopt EctoSearcher.Mapping behaviour and implement callbacks or use EctoSearcher.Mapping, which provides defaults. defmodule CustomMapping do use EctoSearcher.Mapping require Ecto.Query alias Ecto.Query def matchers %{ &quot;not_eq&quot; =&gt; fn(field, value) -&gt; Query.dynamic([q], ^field != ^value) end } end end"},{"ref":"EctoSearcher.Mapping.html#c:fields/0","title":"EctoSearcher.Mapping.fields/0","type":"callback","doc":"Should return map with field queries Field queries map should look like: %{ id_alias: Query.dynamic([q], q.id), datetime_field_as_date: %{ query: Query.dynamic([q], fragment(&quot;?::date&quot;, q.datetime_field)), type: :date } } Field name will be matched as search field prefix (from searchable_fields). Values should either be a Ecto.Query.DynamicExpr or a map with Ecto.Query.DynamicExpr as :query and value type as :type. EctoSearcher.Searcher.search/5 and EctoSearcher.Sorter.sort/5 looks up fields in mapping first, then looks up fields in schema."},{"ref":"EctoSearcher.Mapping.html#c:matchers/0","title":"EctoSearcher.Mapping.matchers/0","type":"callback","doc":"Should return map with search matchers Search matcher map should look like: %{ &quot;not_eq&quot; =&gt; fn(field, value) -&gt; Query.dynamic([q], ^field != ^value) end &quot;in&quot; =&gt; %{ query: fn field, value -&gt; Query.dynamic([q], ^field in ^value) end, aggregation: :array } } Matcher name will be matched as search field suffix. Values should either be a query function or a map with query function as :query and value aggregate type as :aggregation. Query function will be called with arguments field (atom) and value (casted to specific type) and should return Ecto.Query.DynamicExpr."},{"ref":"EctoSearcher.Mapping.Default.html","title":"EctoSearcher.Mapping.Default","type":"module","doc":"Contains default matcher mapping. Matchers eq — equality (field == value) cont — contains substring (ilike(field, value)) in — inclusion (field in value) gt — greater than (field &gt; value) gteq — greater than or equal (field &gt;= value) lt — less than (field &lt; value) lteq — less than or equal (field &lt;= value) overlaps — arrays overlap (field &amp;&amp; value)"},{"ref":"EctoSearcher.Mapping.Default.html#fields/0","title":"EctoSearcher.Mapping.Default.fields/0","type":"function","doc":"Callback implementation for EctoSearcher.Mapping.fields/0"},{"ref":"EctoSearcher.Mapping.Default.html#matchers/0","title":"EctoSearcher.Mapping.Default.matchers/0","type":"function","doc":"Callback implementation for EctoSearcher.Mapping.matchers/0"},{"ref":"EctoSearcher.Searcher.html","title":"EctoSearcher.Searcher","type":"module","doc":"Module for searching Usage search = %{&quot;name_eq&quot; =&gt; &quot;Donald Trump&quot;, &quot;description_cont&quot; =&gt; &quot;My president&quot;} query = EctoSearcher.Searcher.search(MyMegaModel, search) MySuperApp.Repo.all(query)"},{"ref":"EctoSearcher.Searcher.html#search/3","title":"EctoSearcher.Searcher.search/3","type":"function","doc":"Shortcut for search/5 with Default as mapping and nil as searchable_fields"},{"ref":"EctoSearcher.Searcher.html#search/4","title":"EctoSearcher.Searcher.search/4","type":"function","doc":"Shortcut for search/5 with Default as mapping"},{"ref":"EctoSearcher.Searcher.html#search/5","title":"EctoSearcher.Searcher.search/5","type":"function","doc":"Builds search query Takes %Ecto.Query{} as base_query and ecto model as schema. search_params should be a map with search_fields in form of &quot;field_matcher&quot; like this: %{ &quot;name_eq&quot; =&gt; &quot;Donald Trump&quot;, &quot;description_cont&quot; =&gt; &quot;My president&quot; } mapping should implement EctoSearcher.Mapping behavior. EctoSearcher.Mapping.Default provides some basics. searchable_fields is a list with fields (atoms) permitted for searching. If not provided (or nil) all fields are allowed for searching."},{"ref":"EctoSearcher.Sorter.html","title":"EctoSearcher.Sorter","type":"module","doc":"Module for sorting Usage sortable_fields = [:name, :description] sorted_query = EctoSearcher.Sorter.sort(SomeEctoModel, %{&quot;field&quot; =&gt; &quot;name&quot;, &quot;order&quot; =&gt; &quot;desc&quot;}, sortable_fields) MySuperApp.Repo.all(sorted_query)"},{"ref":"EctoSearcher.Sorter.html#sort/3","title":"EctoSearcher.Sorter.sort/3","type":"function","doc":"Shortcut for sort/5"},{"ref":"EctoSearcher.Sorter.html#sort/5","title":"EctoSearcher.Sorter.sort/5","type":"function","doc":"Builds sort query Takes %Ecto.Query{} as base_query and ecto model as schema. sort_params should be a map with &quot;field&quot; and &quot;order&quot; like this: %{ &quot;field&quot; =&gt; &quot;name&quot;, &quot;order&quot; =&gt; &quot;asc&quot; } mapping should implement EctoSearcher.Mapping behavior. EctoSearcher.Mapping.Default provides some basics. sortable_fields is a list with fields (atoms) permitted for sorting. If not provided (or nil) all fields are allowed for sorting: [:name, :description]"},{"ref":"EctoSearcher.Utils.SearchCondition.html","title":"EctoSearcher.Utils.SearchCondition","type":"module","doc":"Builds SearchCondition from params This module is internal. Use at your own risk."},{"ref":"EctoSearcher.Utils.SearchCondition.html#build/2","title":"EctoSearcher.Utils.SearchCondition.build/2","type":"function","doc":"Builds %SearchCondition{} from search expression. Usage searchable_fields = [:name, :description] search_expression = {&quot;name_eq&quot;, &quot;Donald Trump&quot;} EctoSearcher.Searcher.SearchCondition.build(search_expression, searchable_fields) # =&gt; EctoSearcher.Searcher.SearchCondition(field: :name, matcher: &quot;eq&quot;, value: &quot;Donald Trump&quot;)"},{"ref":"EctoSearcher.Utils.SearchCondition.html#from_params/2","title":"EctoSearcher.Utils.SearchCondition.from_params/2","type":"function","doc":"Builds %SearchCondition{} from params Usage searchable_fields = [:name, :description] search_params = %{&quot;name_eq&quot; =&gt; &quot;Donald Trump&quot;, &quot;description_cont&quot; =&gt; &quot;My president&quot;} EctoSearcher.Searcher.SearchCondition.from_params(search_params, searchable_fields) # =&gt; [ # EctoSearcher.Searcher.SearchCondition(field: :name, matcher: &quot;eq&quot;, value: &quot;Donald Trump&quot;), # EctoSearcher.Searcher.SearchCondition(field: :description, matcher: &quot;cont&quot;, value: &quot;My president&quot;), # ]"}]